from db import db, secret_key

from flask_httpauth import HTTPBasicAuth

from passlib.apps import custom_app_context as pwd_context
from itsdangerous import (TimedJSONWebSignatureSerializer as Serializer, BadSignature, SignatureExpired)

auth = HTTPBasicAuth()


class AccountModel(db.Model):
    __tablename__ = 'account'

    id = db.Column(db.Integer, primary_key=True, unique=True)
    rol = db.Column(db.Integer, nullable=False)
    name = db.Column(db.String(30), nullable=False)
    lastname = db.Column(db.String(30), nullable=False)
    email = db.Column(db.String(30), nullable=False)
    password = db.Column(db.String, nullable=False)

    companies = db.relationship('CompanyModel', backref='company_list', lazy=True)
    email_templates = db.relationship('EmailTemplateModel', backref='email_template_list', lazy=True)
    campaigns = db.relationship('CampaignModel', backref='campaign_list', lazy=True)

    def __init__(self, name, lastname, email, password):
        self.name = name
        self.lastname = lastname
        self.email = email
        self.hash_password(password)
        self.rol = 0

    def save_to_db(self):
        db.session.add(self)
        db.session.commit()

    def delete_from_db(self):
        db.session.delete(self)
        db.session.commit()

    def json(self):
        return {"User": {
            'id': self.id,
            'name': self.name,
            'lastname': self.lastname,
            'email': self.email,
            'rol': self.rol,
            'companies': [a.json() for a in self.companies],
            'email_templates': [a.json() for a in self.email_templates],
            'campaigns': [a.json() for a in self.campaigns]
        }}

    @classmethod
    def get_users(cls):
        list_users = [user.json() for user in AccountModel.query.all()]
        dicc = {"users": list_users}
        return dicc

    @classmethod
    def find_by_id(cls, idd):
        return db.session.query(AccountModel).filter_by(id=idd).first()

    @classmethod
    def find_by_email(cls, email):
        return db.session.query(AccountModel).filter_by(email=email).first()

    def hash_password(self, password):
        self.password = pwd_context.encrypt(password)

    def verify_password(self, password):
        return pwd_context.verify(password, self.password)

    def generate_auth_token(self, expiration=3600):
        s = Serializer(secret_key, expires_in=expiration)
        return s.dumps({'email': self.email})

    @classmethod
    def verify_auth_token(cls, token):
        s = Serializer(secret_key)
        try:
            data = s.loads(token)
        except SignatureExpired:
            return None
        except BadSignature:
            return None

        user = cls.query.filter_by(email=data['email']).first()

        return user


def verify_account(idd, token):
    user = AccountModel.verify_auth_token(token)
    if user and user.id == int(idd):
        return user


def get_user_roles(user):
    roles = {
        0: 'client',
        1: 'admin'
    }
    return roles[user.rol]
